#!/usr/bin/env perl
#!/bin/env perl

## Name: getisoforms.pl
## Version: 1.2.2
## Date: Mar 11, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

# get the first gene model
# --input, input gff/gtf file, containing only transcripts and their exons
##e.g.
##transcript
##exon
##exon
##transcript
##exon
# --output, gff/gtf transcript output file

use strict;
use warnings;
use Getopt::Long;

my $help;
my $input;
my $output = "output.gff";
my $line;
my @args;
my @tscargs;
my @exons;
my %hashtsc;
my $tsc;
my $key;
my @note;
my @isoforms;
my $i;

usage() if (@ARGV < 2 or
	    ! GetOptions("help|?" => \$help,
			 "input=s" => \$input,
			 "output=s" => \$output)
	    or defined $help);

open(GFF, $input) or die("Input file not found\n");
open(TSC, ">$output");

$line = <GFF>;
if ($line) {
    chomp($line);
}
else {
    print "Empty file\n";
    exit 0;
}
do {
    #< take transcript
    %hashtsc = ();
    $tsc = $line;
    @tscargs = split("\t", $line);
    @note = split(";", $tscargs[8]);
    #>
    $line = <GFF>;
    if ($line) {
	chomp($line);
	@args = split("\t", $line);
    }
    @isoforms = ();
    #< take exons of the first model .1
    while ($line && $args[2] eq "exon") {
	@args = split("[=;]", $args[8]);
	if (!exists $hashtsc{$args[1]}) {
	    push(@isoforms, $args[1]);
	}
	push(@{$hashtsc{$args[1]}}, "$line\n");
	$line = <GFF>;
	if ($line) {
	    chomp($line);
	    @args = split("\t", $line);
	}
    }
    #>
    
    foreach (@isoforms) {
      	for ($i=0; $i<$#tscargs; $i++) {
	    print TSC "$tscargs[$i]\t";
	}
	$note[0] = "ID=$_";
	for ($i=0; $i<$#note; $i++) {
            print TSC "$note[$i];";
        }
	print TSC "$note[$i]\n";
	for ($i=0; $i<=$#{$hashtsc{$_}}; $i++) {
            print TSC "${$hashtsc{$_}}[$i]";
        }
    }
} until (!$line);
close(GFF);
close(TSC);

#<sort transcript file
##system("sort -k1,1 -k4,4n -k5,5n $transcript > sort_$transcript");
##system("mv sort_$transcript $transcript");
#>

exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl getisoforms.pl --input GFF [--output TSC] [--help|?] \n";
    print "Notice: input file should only contain transcript and exon features\n";
    exit 1;
}
