#!/usr/bin/env perl
#!/bin/env perl

## Name: rmwrongpairmap.pl
## Version: 1.2.2
## Date: Mar 11, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

## remove wrongly paired mapping from id sorted sam file
# --input, input sam file
# --max, max insert size
# --output, output sam file

use strict;
use warnings;
use Getopt::Long;

my $help;
my $input;
my $output;
my $max = 2000;
my $line1;
my $line2;
my @args;
my $id;
my $chr;

usage() if (@ARGV < 4 or
	    ! GetOptions("help|?" => \$help,
			 "input=s" => \$input,
			 "max=i" => \$max,
			 "output=s" => \$output)
	    or defined $help);

if (defined $input) {
    open(STDIN, $input);
}
if (defined $output) {
    open(STDOUT, ">$output");
}

while ($line1 = <STDIN>) {
    chomp($line1);
    @args = split("\t", $line1);
    if ($args[0] eq "\@HD" || $args[0] eq "\@SQ" || $args[0] eq "\@PG" || $args[0] eq "\@RG" || $args[0] eq "\@CO") {
        print STDOUT "$line1\n";
    }
    else {
        last;
    }
}
$id = $args[0];
$chr = $args[2];
while ($line2 = <STDIN>) {
    chomp($line2);
    @args = split("\t", $line2);
    #-- satisfy pairing
    if ($id eq $args[0] && $chr eq $args[2]) {
	#-- do not consider maximum insert size, as the SAM flags are taken into account in separatestrand.pl
	#-- uncomment the 'if' statement if the consideration of maximum insert size is required
	#if (abs($args[8]) < $max) {
	    print STDOUT "$line1\n";
	    print STDOUT "$line2\n";
	#}
	$line1 = <STDIN>;
	if (!$line1) {
	    last;
	}
	chomp($line1);
	@args = split("\t", $line1);
    }
    else {
	$line1 = $line2;
    }
    $id = $args[0];
    $chr = $args[2];
}
close(STDIN);
close(STDOUT);
exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl rmwrongpairmap.pl --input SAM --max MAX [--output OUT] [--help|?] \n";
    exit 1;
}
